@@ -1,54 +1,143 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright © 2009-2010 CEA
-# Pierre Raybaut
-# Licensed under the terms of the CECILL License
-# (see guidata/__init__.py for details)
+# Licensed under the terms of the BSD 3-Clause
+# (see guidata/LICENSE for details)
 
 """
-dataset.dataitems
-=================
+Data items
+----------
 
-The ``guidata.dataset.dataitems`` module contains implementation for 
-concrete DataItems.
+Numeric items
+^^^^^^^^^^^^^
+
+.. autoclass:: FloatItem
+    :members:
+
+.. autoclass:: IntItem
+    :members:
+
+.. autoclass:: FloatArrayItem
+    :members:
+
+Text items
+^^^^^^^^^^
+
+.. autoclass:: StringItem
+    :members:
+
+.. autoclass:: TextItem
+    :members:
+
+Date and time items
+^^^^^^^^^^^^^^^^^^^
+
+.. autoclass:: DateItem
+    :members:
+
+.. autoclass:: DateTimeItem
+    :members:
+
+Color items
+^^^^^^^^^^^
+
+.. autoclass:: ColorItem
+    :members:
+
+File items
+^^^^^^^^^^
+
+.. autoclass:: FileSaveItem
+    :members:
+
+.. autoclass:: FileOpenItem
+    :members:
+
+.. autoclass:: FilesOpenItem
+    :members:
+
+.. autoclass:: DirectoryItem
+    :members:
+
+Choice items
+^^^^^^^^^^^^
+
+.. autoclass:: BoolItem
+    :members:
+
+.. autoclass:: ChoiceItem
+    :members:
+
+.. autoclass:: MultipleChoiceItem
+    :members:
+
+.. autoclass:: ImageChoiceItem
+    :members:
+
+Other items
+^^^^^^^^^^^
+
+.. autoclass:: ButtonItem
+    :members:
+
+.. autoclass:: DictItem
+    :members:
+
+.. autoclass:: FontFamilyItem
+    :members:
 """
 
-from __future__ import division, unicode_literals
-
+import collections.abc
+import datetime
 import os
 import re
-import datetime
-import collections
+from typing import TYPE_CHECKING, Any, Callable, List, Optional, Tuple, Type, Union
 
-from guidata.dataset.datatypes import DataItem, ItemProperty
-from guidata.utils import utf8_to_unicode, add_extension
 from guidata.config import _
-from qtpy.py3compat import to_text_string, is_text_string, TEXT_TYPES
+from guidata.dataset.datatypes import DataItem, DataSet, ItemProperty
+from guidata.dataset.iniio import UserConfigReader, UserConfigWriter
+from guidata.qthelpers import exec_dialog
+from numpy import ndarray
+
+if TYPE_CHECKING:  # pragma: no cover
+    from guidata.dataset.hdf5io import HDF5Reader, HDF5Writer
+    from guidata.dataset.jsonio import JSONReader, JSONWriter
 
 
 class NumericTypeItem(DataItem):
     """
     Numeric data item
+    * label [string]: name
+    * default [int]: default value (optional)
+    * min [int]: minimum value (optional)
+    * max [int]: maximum value (optional)
+    * nonzero [bool]: if True, zero is not a valid value (optional)
+    * unit [string]: physical unit (optional)
+    * even [bool]: if True, even values are valid, if False,
+        odd values are valid if None (default), ignored (optional)
+    * slider [bool]: if True, shows a slider widget right after the line
+        edit widget (default is False)
+    * help [string]: text shown in tooltip (optional)
+    * check [bool]: if False, value is not checked (optional, default=True)
     """
 
-    type = None
+    type: Callable = None
 
     def __init__(
         self,
-        label,
-        default=None,
-        min=None,
-        max=None,
-        nonzero=None,
-        unit="",
-        help="",
-        check=True,
-    ):
+        label: str,
+        default: Optional[Union[float, int]] = None,
+        min: Optional[Union[float, int]] = None,
+        max: Optional[Union[float, int]] = None,
+        nonzero: Optional[bool] = None,
+        unit: str = "",
+        help: str = "",
+        check: bool = True,
+    ) -> None:
         DataItem.__init__(self, label, default=default, help=help)
         self.set_prop("data", min=min, max=max, nonzero=nonzero, check_value=check)
         self.set_prop("display", unit=unit)
 
-    def get_auto_help(self, instance):
+    def get_auto_help(self, instance: DataSet) -> str:
         """Override DataItem method"""
         auto_help = {int: _("integer"), float: _("float")}[self.type]
         _min = self.get_prop_value("data", instance, "min")
@@ -67,7 +156,9 @@ class NumericTypeItem(DataItem):
             auto_help += ", %s %s" % (_("unit:"), unit)
         return auto_help
 
-    def format_string(self, instance, value, fmt, func):
+    def format_string(
+        self, instance: "DataSet", value: Union[float, int], fmt: str, func: Callable
+    ) -> str:
         """Override DataItem method"""
         text = fmt % (func(value),)
         # We add directly the unit to 'text' (instead of adding it
@@ -77,7 +168,7 @@ class NumericTypeItem(DataItem):
             text += " " + unit
         return text
 
-    def check_value(self, value):
+    def check_value(self, value: Union[float, int]) -> bool:
         """Override DataItem method"""
         if not self.get_prop("data", "check_value", True):
             return True
@@ -95,9 +186,8 @@ class NumericTypeItem(DataItem):
                 return False
         return True
 
-    def from_string(self, value):
+    def from_string(self, value: str) -> Optional[Any]:
         """Override DataItem method"""
-        value = to_text_string(value)  # necessary if value is a QString
         # String may contains numerical operands:
         if re.match(r"^([\d\(\)\+/\-\*.]|e)+$", value):
             try:
@@ -127,18 +217,18 @@ class FloatItem(NumericTypeItem):
 
     def __init__(
         self,
-        label,
-        default=None,
-        min=None,
-        max=None,
-        nonzero=None,
-        unit="",
-        step=0.1,
-        slider=False,
-        help="",
-        check=True,
-    ):
-        super(FloatItem, self).__init__(
+        label: str,
+        default: Optional[float] = None,
+        min: Optional[float] = None,
+        max: Optional[float] = None,
+        nonzero: Optional[bool] = None,
+        unit: str = "",
+        step: float = 0.1,
+        slider: bool = False,
+        help: str = "",
+        check: bool = True,
+    ) -> None:
+        super().__init__(
             label,
             default=default,
             min=min,
@@ -151,7 +241,9 @@ class FloatItem(NumericTypeItem):
         self.set_prop("display", slider=slider)
         self.set_prop("data", step=step)
 
-    def get_value_from_reader(self, reader):
+    def get_value_from_reader(
+        self, reader: Union["HDF5Reader", "JSONReader", "UserConfigReader"]
+    ) -> float:
         """Reads value from the reader object, inside the try...except
         statement defined in the base item `deserialize` method"""
         return reader.read_float()
@@ -178,18 +270,18 @@ class IntItem(NumericTypeItem):
 
     def __init__(
         self,
-        label,
-        default=None,
-        min=None,
-        max=None,
-        nonzero=None,
-        unit="",
-        even=None,
-        slider=False,
-        help="",
-        check=True,
-    ):
-        super(IntItem, self).__init__(
+        label: str,
+        default: Optional[int] = None,
+        min: Optional[int] = None,
+        max: Optional[int] = None,
+        nonzero: Optional[bool] = None,
+        unit: str = "",
+        even: Optional[bool] = None,
+        slider: bool = False,
+        help: str = "",
+        check: bool = True,
+    ) -> None:
+        super().__init__(
             label,
             default=default,
             min=min,
@@ -202,9 +294,9 @@ class IntItem(NumericTypeItem):
         self.set_prop("data", even=even)
         self.set_prop("display", slider=slider)
 
-    def get_auto_help(self, instance):
+    def get_auto_help(self, instance: "DataSet") -> str:
         """Override DataItem method"""
-        auto_help = super(IntItem, self).get_auto_help(instance)
+        auto_help = super().get_auto_help(instance)
         even = self.get_prop_value("data", instance, "even")
         if even is not None:
             if even:
@@ -213,11 +305,11 @@ class IntItem(NumericTypeItem):
                 auto_help += ", " + _("odd")
         return auto_help
 
-    def check_value(self, value):
+    def check_value(self, value: Any) -> bool:
         """Override DataItem method"""
         if not self.get_prop("data", "check_value", True):
             return True
-        valid = super(IntItem, self).check_value(value)
+        valid = super().check_value(value)
         if not valid:
             return False
         even = self.get_prop("data", "even")
@@ -227,7 +319,9 @@ class IntItem(NumericTypeItem):
                 return False
         return True
 
-    def get_value_from_reader(self, reader):
+    def get_value_from_reader(
+        self, reader: Union["HDF5Reader", "JSONReader", "UserConfigReader"]
+    ) -> Any:
         """Reads value from the reader object, inside the try...except
         statement defined in the base item `deserialize` method"""
         return reader.read_int()
@@ -243,26 +337,34 @@ class StringItem(DataItem):
         * wordwrap [bool]: toggle word wrapping (optional)
     """
 
-    type = TEXT_TYPES
+    type: Any = str
 
-    def __init__(self, label, default=None, notempty=None, wordwrap=False, help=""):
+    def __init__(
+        self,
+        label: str,
+        default: Optional[str] = None,
+        notempty: Optional[bool] = None,
+        wordwrap: bool = False,
+        help: str = "",
+    ) -> None:
         DataItem.__init__(self, label, default=default, help=help)
         self.set_prop("data", notempty=notempty)
         self.set_prop("display", wordwrap=wordwrap)
 
-    def check_value(self, value):
+    def check_value(self, value: Any) -> bool:
         """Override DataItem method"""
         notempty = self.get_prop("data", "notempty")
         if notempty and not value:
             return False
         return True
 
-    def from_string(self, value):
+    def from_string(self, value: str) -> str:
         """Override DataItem method"""
-        # QString -> str
-        return to_text_string(value)
+        return value
 
-    def get_value_from_reader(self, reader):
+    def get_value_from_reader(
+        self, reader: Union["HDF5Reader", "JSONReader", "UserConfigReader"]
+    ) -> Any:
         """Reads value from the reader object, inside the try...except
         statement defined in the base item `deserialize` method"""
         return reader.read_unicode()
@@ -278,7 +380,14 @@ class TextItem(StringItem):
         * wordwrap [bool]: toggle word wrapping (optional)
     """
 
-    def __init__(self, label, default=None, notempty=None, wordwrap=True, help=""):
+    def __init__(
+        self,
+        label: str,
+        default: Optional[str] = None,
+        notempty: Optional[bool] = None,
+        wordwrap: bool = True,
+        help: str = "",
+    ) -> None:
         StringItem.__init__(
             self,
             label,
@@ -301,11 +410,20 @@ class BoolItem(DataItem):
 
     type = bool
 
-    def __init__(self, text="", label="", default=None, help="", check=True):
+    def __init__(
+        self,
+        text: str = "",
+        label: str = "",
+        default: Optional[bool] = None,
+        help: str = "",
+        check: bool = True,
+    ) -> None:
         DataItem.__init__(self, label, default=default, help=help, check=check)
         self.set_prop("display", text=text)
 
-    def get_value_from_reader(self, reader):
+    def get_value_from_reader(
+        self, reader: Union["HDF5Reader", "JSONReader", "UserConfigReader"]
+    ) -> bool:
         """Reads value from the reader object, inside the try...except
         statement defined in the base item `deserialize` method"""
         return reader.read_bool()
@@ -324,6 +442,14 @@ class DateItem(DataItem):
 
 
 class DateTimeItem(DateItem):
+    """
+    Construct a date time data item.
+        * text [string]: form's field name (optional)
+        * label [string]: name
+        * default [datetime.date]: default value (optional)
+        * help [string]: text shown in tooltip (optional)
+    """
+
     pass
 
 
@@ -338,7 +464,7 @@ class ColorItem(StringItem):
     Color values are encoded as hexadecimal strings or Qt color names
     """
 
-    def check_value(self, value):
+    def check_value(self, value: str) -> bool:
         """Override DataItem method"""
         if not self.get_prop("data", "check_value", True):
             return True
@@ -348,7 +474,9 @@ class ColorItem(StringItem):
 
         return text_to_qcolor(value).isValid()
 
-    def get_value_from_reader(self, reader):
+    def get_value_from_reader(
+        self, reader: Union["HDF5Reader", "JSONReader", "UserConfigReader"]
+    ) -> str:
         """Reads value from the reader object, inside the try...except
         statement defined in the base item `deserialize` method"""
         # Using read_str converts `numpy.string_` to `str` -- otherwise,
@@ -370,22 +498,22 @@ class FileSaveItem(StringItem):
 
     def __init__(
         self,
-        label,
-        formats="*",
-        default=None,
-        basedir=None,
-        all_files_first=False,
-        help="",
-        check=True,
-    ):
+        label: str,
+        formats: Union[Tuple[str, ...], str] = "*",
+        default: Optional[Union[List[str], str]] = None,
+        basedir: Optional[str] = None,
+        all_files_first: bool = False,
+        help: str = "",
+        check: bool = True,
+    ) -> None:
         DataItem.__init__(self, label, default=default, help=help, check=check)
         if isinstance(formats, str):
-            formats = [formats]
+            formats = [formats]  # type:ignore
         self.set_prop("data", formats=formats)
         self.set_prop("data", basedir=basedir)
         self.set_prop("data", all_files_first=all_files_first)
 
-    def get_auto_help(self, instance):
+    def get_auto_help(self, instance: "DataSet") -> str:
         """Override DataItem method"""
         formats = self.get_prop("data", "formats")
         return (
@@ -394,7 +522,7 @@ class FileSaveItem(StringItem):
             else _("supported file types:") + " *.%s" % ", *.".join(formats)
         )
 
-    def check_value(self, value):
+    def check_value(self, value: str) -> bool:
         """Override DataItem method"""
         if not self.get_prop("data", "check_value", True):
             return True
@@ -402,9 +530,19 @@ class FileSaveItem(StringItem):
             return False
         return len(value) > 0
 
-    def from_string(self, value):
+    def from_string(self, value) -> str:
         """Override DataItem method"""
-        return add_extension(self, value)
+        return self.add_extension(value)
+
+    def add_extension(self, value) -> str:
+        """Add extension to filename
+        `value`: possible value for data item"""
+        value = str(value)
+        formats = self.get_prop("data", "formats")
+        if len(formats) == 1 and formats[0] != "*":
+            if not value.endswith("." + formats[0]) and len(value) > 0:
+                return value + "." + formats[0]
+        return value
 
 
 class FileOpenItem(FileSaveItem):
@@ -418,7 +556,7 @@ class FileOpenItem(FileSaveItem):
         * check [bool]: if False, value is not checked (optional, default=True)
     """
 
-    def check_value(self, value):
+    def check_value(self, value: str) -> bool:
         """Override DataItem method"""
         if not self.get_prop("data", "check_value", True):
             return True
@@ -442,15 +580,15 @@ class FilesOpenItem(FileSaveItem):
 
     def __init__(
         self,
-        label,
-        formats="*",
-        default=None,
-        basedir=None,
-        all_files_first=False,
-        help="",
-        check=True,
-    ):
-        if is_text_string(default):
+        label: str,
+        formats: str = "*",
+        default: Optional[Union[List[str], str]] = None,
+        basedir: Optional[str] = None,
+        all_files_first: bool = False,
+        help: str = "",
+        check: bool = True,
+    ) -> None:
+        if isinstance(default, str):
             default = [default]
         FileSaveItem.__init__(
             self,
@@ -463,7 +601,7 @@ class FilesOpenItem(FileSaveItem):
             check=check,
         )
 
-    def check_value(self, value):
+    def check_value(self, value: str) -> bool:
         """Override DataItem method"""
         if not self.get_prop("data", "check_value", True):
             return True
@@ -474,24 +612,29 @@ class FilesOpenItem(FileSaveItem):
             allexist = allexist and os.path.exists(path) and os.path.isfile(path)
         return allexist
 
-    def from_string(self, value):
+    def from_string(self, value: Any) -> List[str]:  # type:ignore
         """Override DataItem method"""
-        value = to_text_string(value)
         if value.endswith("']") or value.endswith('"]'):
             value = eval(value)
         else:
             value = [value]
-        return [add_extension(self, path) for path in value]
+        return [self.add_extension(path) for path in value]
 
-    def serialize(self, instance, writer):
+    def serialize(
+        self,
+        instance: "DataSet",
+        writer: Union["HDF5Writer", "JSONWriter", "UserConfigWriter"],
+    ) -> None:
         """Serialize this item"""
         value = self.get_value(instance)
         writer.write_sequence([fname.encode("utf-8") for fname in value])
 
-    def get_value_from_reader(self, reader):
+    def get_value_from_reader(
+        self, reader: Union["HDF5Reader", "JSONReader", "UserConfigReader"]
+    ) -> List[str]:
         """Reads value from the reader object, inside the try...except
         statement defined in the base item `deserialize` method"""
-        return [to_text_string(fname, "utf-8") for fname in reader.read_sequence()]
+        return [fname for fname in reader.read_sequence()]
 
 
 class DirectoryItem(StringItem):
@@ -503,7 +646,7 @@ class DirectoryItem(StringItem):
         * check [bool]: if False, value is not checked (optional, default=True)
     """
 
-    def check_value(self, value):
+    def check_value(self, value: str) -> bool:
         """Override DataItem method"""
         if not self.get_prop("data", "check_value", True):
             return True
@@ -512,7 +655,12 @@ class DirectoryItem(StringItem):
         return os.path.exists(value) and os.path.isdir(value)
 
 
-class FirstChoice(object):
+class FirstChoice:
+    """
+    Special object that means the default value of a ChoiceItem
+    is the first item.
+    """
+
     pass
 
 
@@ -532,15 +680,24 @@ class ChoiceItem(DataItem):
     """
 
     def __init__(
-        self, label, choices, default=FirstChoice, help="", check=True, radio=False
-    ):
-        if isinstance(choices, collections.Callable):
+        self,
+        label: str,
+        choices: Any,
+        default: Optional[Union[Tuple[()], Type[FirstChoice], int]] = FirstChoice,
+        help: str = "",
+        check: bool = True,
+        radio: bool = False,
+    ) -> None:
+        _choices_data: Any
+        if isinstance(choices, collections.abc.Callable):  # type:ignore
             _choices_data = ItemProperty(choices)
         else:
             _choices_data = []
             for idx, choice in enumerate(choices):
                 _choices_data.append(self._normalize_choice(idx, choice))
-        if default is FirstChoice and not isinstance(choices, collections.Callable):
+        if default is FirstChoice and not isinstance(
+            choices, collections.abc.Callable  # type:ignore
+        ):
             default = _choices_data[0][0]
         elif default is FirstChoice:
             default = None
@@ -548,31 +705,24 @@ class ChoiceItem(DataItem):
         self.set_prop("data", choices=_choices_data)
         self.set_prop("display", radio=radio)
 
-    def _normalize_choice(self, idx, choice_tuple):
+    def _normalize_choice(
+        self, idx: int, choice_tuple: Tuple[Any, ...]
+    ) -> Union[Tuple[int, str, None], Tuple[str, str, None]]:
         if isinstance(choice_tuple, tuple):
             key, value = choice_tuple
         else:
             key = idx
             value = choice_tuple
-
-        if isinstance(value, str):
-            value = utf8_to_unicode(value)
         return (key, value, None)
 
-    #    def _choices(self, item):
-    #        _choices_data = self.get_prop("data", "choices")
-    #        if callable(_choices_data):
-    #            return _choices_data(self, item)
-    #        return _choices_data
-
-    def get_string_value(self, instance):
+    def get_string_value(self, instance: "DataSet") -> str:
         """Override DataItem method"""
         value = self.get_value(instance)
         choices = self.get_prop_value("data", instance, "choices")
         # print "ShowChoiceWidget:", choices, value
         for choice in choices:
             if choice[0] == value:
-                return to_text_string(choice[1])
+                return str(choice[1])
         else:
             return DataItem.get_string_value(self, instance)
 
@@ -587,11 +737,18 @@ class MultipleChoiceItem(ChoiceItem):
         * check [bool]: if False, value is not checked (optional, default=True)
     """
 
-    def __init__(self, label, choices, default=(), help="", check=True):
+    def __init__(
+        self,
+        label: str,
+        choices: List[str],
+        default: Tuple[()] = (),
+        help: str = "",
+        check: bool = True,
+    ) -> None:
         ChoiceItem.__init__(self, label, choices, default, help, check=check)
         self.set_prop("display", shape=(1, -1))
 
-    def horizontal(self, row_nb=1):
+    def horizontal(self, row_nb: int = 1) -> "MultipleChoiceItem":
         """
         Method to arange choice list horizontally on `n` rows
 
@@ -601,7 +758,7 @@ class MultipleChoiceItem(ChoiceItem):
         self.set_prop("display", shape=(row_nb, -1))
         return self
 
-    def vertical(self, col_nb=1):
+    def vertical(self, col_nb: int = 1) -> "MultipleChoiceItem":
         """
         Method to arange choice list vertically on `n` columns
 
@@ -611,7 +768,11 @@ class MultipleChoiceItem(ChoiceItem):
         self.set_prop("display", shape=(-1, col_nb))
         return self
 
-    def serialize(self, instance, writer):
+    def serialize(
+        self,
+        instance: "DataSet",
+        writer: Union["HDF5Writer", "JSONWriter", "UserConfigWriter"],
+    ) -> None:
         """Serialize this item"""
         value = self.get_value(instance)
         seq = []
@@ -620,17 +781,25 @@ class MultipleChoiceItem(ChoiceItem):
             seq.append(key in value)
         writer.write_sequence(seq)
 
-    def deserialize(self, instance, reader):
+    def deserialize(
+        self,
+        instance: "DataSet",
+        reader: Union["HDF5Reader", "JSONReader", "UserConfigReader"],
+    ) -> None:
         """Deserialize this item"""
-        flags = reader.read_sequence()
-        # We could have trouble with objects providing their own choice
-        # function which depend on not yet deserialized values
-        _choices = self.get_prop_value("data", instance, "choices")
-        value = []
-        for idx, flag in enumerate(flags):
-            if flag:
-                value.append(_choices[idx][0])
-        self.__set__(instance, value)
+        try:
+            flags = reader.read_sequence()
+        except KeyError:
+            self.set_default(instance)
+        else:
+            # We could have trouble with objects providing their own choice
+            # function which depend on not yet deserialized values
+            _choices = self.get_prop_value("data", instance, "choices")
+            value = []
+            for idx, flag in enumerate(flags):
+                if flag:
+                    value.append(_choices[idx][0])
+            self.__set__(instance, value)
 
 
 class ImageChoiceItem(ChoiceItem):
@@ -647,16 +816,15 @@ class ImageChoiceItem(ChoiceItem):
         * check [bool]: if False, value is not checked (optional, default=True)
     """
 
-    def _normalize_choice(self, idx, choice_tuple):
+    def _normalize_choice(
+        self, idx: int, choice_tuple: Tuple[Any, ...]
+    ) -> Tuple[Any, Any, Any]:
         assert isinstance(choice_tuple, tuple)
         if len(choice_tuple) == 3:
             key, value, img = choice_tuple
         else:
             key = idx
             value, img = choice_tuple
-
-        if isinstance(value, str):
-            value = utf8_to_unicode(value)
         return (key, value, img)
 
 
@@ -675,18 +843,20 @@ class FloatArrayItem(DataItem):
 
     def __init__(
         self,
-        label,
-        default=None,
-        help="",
-        format="%.3f",
-        transpose=False,
-        minmax="all",
-        check=True,
-    ):
+        label: str,
+        default: Optional[ndarray] = None,
+        help: str = "",
+        format: str = "%.3f",
+        transpose: bool = False,
+        minmax: str = "all",
+        check: bool = True,
+    ) -> None:
         DataItem.__init__(self, label, default=default, help=help, check=check)
         self.set_prop("display", format=format, transpose=transpose, minmax=minmax)
 
-    def format_string(self, instance, value, fmt, func):
+    def format_string(
+        self, instance: "DataSet", value: Any, fmt: str, func: Callable
+    ) -> str:
         """Override DataItem method"""
         larg = self.get_prop_value("display", instance, "large", False)
         fmt = self.get_prop_value("display", instance, "format", "%s")
@@ -703,14 +873,20 @@ class FloatArrayItem(DataItem):
             text += " .. " + fmt % v.max()
             text += "]"
         text += " %s" % unit
-        return to_text_string(text)
+        return str(text)
 
-    def serialize(self, instance, writer):
+    def serialize(
+        self,
+        instance: "DataSet",
+        writer: Union["HDF5Writer", "JSONWriter", "UserConfigWriter"],
+    ) -> None:
         """Serialize this item"""
         value = self.get_value(instance)
         writer.write_array(value)
 
-    def get_value_from_reader(self, reader):
+    def get_value_from_reader(
+        self, reader: Union["HDF5Reader", "JSONReader", "UserConfigReader"]
+    ) -> Any:
         """Reads value from the reader object, inside the try...except
         statement defined in the base item `deserialize` method"""
         return reader.read_array()
@@ -736,15 +912,31 @@ class ButtonItem(DataItem):
     with the whole dataset. The value is assigned the callback`s return value.
     """
 
-    def __init__(self, label, callback, icon=None, default=None, help="", check=True):
+    def __init__(
+        self,
+        label: str,
+        callback: Callable,
+        icon: Optional[str] = None,
+        default: Optional[Any] = None,
+        help: str = "",
+        check: bool = True,
+    ) -> None:
         DataItem.__init__(self, label, default=default, help=help, check=check)
         self.set_prop("display", callback=callback)
         self.set_prop("display", icon=icon)
 
-    def serialize(self, instance, writer):
+    def serialize(
+        self,
+        instance: "DataSet",
+        writer: Union["HDF5Writer", "JSONWriter", "UserConfigWriter"],
+    ) -> Any:
         pass
 
-    def deserialize(self, instance, reader):
+    def deserialize(
+        self,
+        instance: "DataSet",
+        reader: Union["HDF5Reader", "JSONReader", "UserConfigReader"],
+    ) -> Any:
         pass
 
 
@@ -757,14 +949,24 @@ class DictItem(ButtonItem):
         * check [bool]: if False, value is not checked (optional, default=True)
     """
 
-    def __init__(self, label, default=None, help="", check=True):
-        def dictedit(instance, item, value, parent):
+    def __init__(
+        self,
+        label: str,
+        default: Optional[dict] = None,
+        help: str = "",
+        check: bool = True,
+    ) -> None:
+        def dictedit(
+            instance: "DataSet", item: "DataItem", value: Any, parent: object
+        ) -> Any:
+            from guidata.widgets.collectionseditor import CollectionsEditor
+
             editor = CollectionsEditor(parent)
             value_was_none = value is None
             if value_was_none:
                 value = {}
             editor.setup(value)
-            if editor.exec_():
+            if exec_dialog(editor):
                 return editor.get_value()
             else:
                 if value_was_none:
