@@ -7,7 +7,13 @@
 # ----------------------------------------------------------------------------
 
 """
-Collections (i.e. dictionary, list and tuple) editor widget and dialog.
+guidata.widgets.collectionseditor
+=================================
+
+This package provides a Collections (i.e. dictionary, list and tuple) editor widget and dialog.
+
+.. autoclass:: CollectionsEditor
+
 """
 
 # TODO: Multiple selection: open as many editors (array/dict/...) as necessary,
@@ -19,8 +25,6 @@ Collections (i.e. dictionary, list and tuple) editor widget and dialog.
 # pylint: disable=R0201
 
 
-from __future__ import print_function
-
 import datetime
 import io
 import re
@@ -28,66 +32,29 @@ import sys
 import warnings
 
 import PIL.Image
-
-from guidata.utils import getcwd_or_home
-from guidata.configtools import get_font, get_icon
-from guidata.qthelpers import add_actions, create_action, mimedata2url
 from guidata.config import CONF, _
-from qtpy.QtWidgets import (
-    QAbstractItemDelegate,
-    QApplication,
-    QDateEdit,
-    QDateTimeEdit,
-    QDialog,
-    QHBoxLayout,
-    QInputDialog,
-    QItemDelegate,
-    QLineEdit,
-    QMenu,
-    QMessageBox,
-    QPushButton,
-    QTableView,
-    QVBoxLayout,
-    QWidget,
-)
-from qtpy.QtCore import (
-    QAbstractTableModel,
-    QLocale,
-    QModelIndex,
-    Qt,
-    Signal,
-    Slot,
-    QDateTime,
-)
-from qtpy.QtGui import (
-    QColor,
-    QKeySequence,
-)
+from guidata.configtools import get_font, get_icon
+from guidata.qthelpers import (add_actions, create_action, mimedata2url,
+                               win32_fix_title_bar_background)
+from guidata.utils.misc import getcwd_or_home
 from guidata.widgets.importwizard import ImportWizard
-from guidata.widgets.nsview import (
-    DataFrame,
-    DatetimeIndex,
-    FakeObject,
-    Image,
-    MaskedArray,
-    Series,
-    array,
-    display_to_value,
-    get_color_name,
-    get_human_readable_type,
-    get_object_attrs,
-    get_size,
-    get_type_string,
-    is_editable_type,
-    is_known_type,
-    ndarray,
-    np_savetxt,
-    sort_against,
-    try_to_eval,
-    unsorted_unique,
-    value_to_display,
-)
+from guidata.widgets.nsview import (DataFrame, DatetimeIndex, FakeObject,
+                                    Image, MaskedArray, Series, array,
+                                    display_to_value, get_color_name,
+                                    get_human_readable_type, get_object_attrs,
+                                    get_size, get_type_string,
+                                    is_editable_type, is_known_type, ndarray,
+                                    np_savetxt, sort_against, try_to_eval,
+                                    unsorted_unique, value_to_display)
 from guidata.widgets.texteditor import TextEditor
+from qtpy.compat import getsavefilename
+from qtpy.QtCore import (QAbstractTableModel, QDateTime, QModelIndex, Qt,
+                         Signal, Slot)
+from qtpy.QtGui import QColor, QKeySequence
+from qtpy.QtWidgets import (QAbstractItemDelegate, QApplication, QDateEdit,
+                            QDateTimeEdit, QDialog, QHBoxLayout, QInputDialog,
+                            QItemDelegate, QLineEdit, QMenu, QMessageBox,
+                            QPushButton, QTableView, QVBoxLayout, QWidget)
 
 if ndarray is not FakeObject:
     from guidata.widgets.arrayeditor import ArrayEditor
@@ -119,7 +86,7 @@ def fix_reference_name(name, blacklist=None):
     return name
 
 
-class ProxyObject(object):
+class ProxyObject:
     """Dictionary proxy to an unknown object."""
 
     def __init__(self, obj):
@@ -468,7 +435,6 @@ class CollectionsModel(ReadOnlyCollectionsModel):
 class CollectionsDelegate(QItemDelegate):
     """CollectionsEditor Item Delegate"""
 
-    sig_free_memory = Signal()
 
     def __init__(self, parent=None):
         QItemDelegate.__init__(self, parent)
@@ -691,7 +657,8 @@ class CollectionsDelegate(QItemDelegate):
             conv_func = data.get("conv", lambda v: v)
             self.set_value(index, conv_func(value))
         self._editors.pop(editor_id)
-        self.free_memory()
+        editor = self.sender()
+        editor.deleteLater()
 
     def editor_rejected(self, editor_id):
         """
@@ -699,14 +666,8 @@ class CollectionsDelegate(QItemDelegate):
         :param editor_id:
         """
         self._editors.pop(editor_id)
-        self.free_memory()
-
-    def free_memory(self):
-        """Free memory after closing an editor."""
-        try:
-            self.sig_free_memory.emit()
-        except RuntimeError:
-            pass
+        editor = self.sender()
+        editor.deleteLater()
 
     def commitAndCloseEditor(self):
         """Overriding method commitAndCloseEditor"""
@@ -793,7 +754,6 @@ class BaseTableView(QTableView):
     sig_option_changed = Signal(str, object)
     sig_files_dropped = Signal(list)
     redirect_stdio = Signal(bool)
-    sig_free_memory = Signal()
 
     def __init__(self, parent):
         QTableView.__init__(self, parent)
@@ -960,10 +920,6 @@ class BaseTableView(QTableView):
         """Return array's ndim"""
         raise NotImplementedError
 
-    def oedit(self, key):
-        """Edit item"""
-        raise NotImplementedError
-
     def plot(self, key, funcname):
         """Plot item"""
         raise NotImplementedError
@@ -1214,7 +1170,7 @@ class BaseTableView(QTableView):
 
     def __prepare_plot(self):
         try:
-            import plotpy.gui.widgets.pyplot  # analysis:ignore
+            import guiqwt.pyplot  # analysis:ignore
 
             return True
         except:
@@ -1222,7 +1178,7 @@ class BaseTableView(QTableView):
                 if "matplotlib" not in sys.modules:
                     import matplotlib
 
-                    matplotlib.use("Qt4Agg")
+                    matplotlib.use("Qt5Agg")
                 return True
             except:
                 QMessageBox.warning(
@@ -1272,7 +1228,7 @@ class BaseTableView(QTableView):
         if self.array_filename is None:
             self.array_filename = getcwd_or_home()
         self.redirect_stdio.emit(False)
-        filename, _selfilter = get_save_filename(
+        filename, _selfilter = getsavefilename(
             self, title, self.array_filename, _("NumPy arrays") + " (*.npy)"
         )
         self.redirect_stdio.emit(True)
@@ -1348,7 +1304,7 @@ class BaseTableView(QTableView):
             contents_title=_("Clipboard contents"),
             varname=fix_reference_name("data", blacklist=list(data.keys())),
         )
-        if editor.exec_():
+        if editor.exec():
             var_name, clip_data = editor.get_data()
             self.new_value(var_name, clip_data)
 
@@ -1449,17 +1405,10 @@ class CollectionsEditorTableView(BaseTableView):
         data = self.model.get_data()
         return data[key].ndim
 
-    def oedit(self, key):
-        """Edit item"""
-        data = self.model.get_data()
-        from plotpy.gui.widgets.variableexplorer.objecteditor import oedit
-
-        oedit(data[key])
-
     def plot(self, key, funcname):
         """Plot item"""
         data = self.model.get_data()
-        import plotpy.gui.widgets.pyplot as plt
+        import guiqwt.pyplot as plt
 
         plt.figure()
         getattr(plt, funcname)(data[key])
@@ -1468,7 +1417,7 @@ class CollectionsEditorTableView(BaseTableView):
     def imshow(self, key):
         """Show item's image"""
         data = self.model.get_data()
-        import plotpy.gui.widgets.pyplot as plt
+        import guiqwt.pyplot as plt
 
         plt.figure()
         plt.imshow(data[key])
@@ -1526,6 +1475,8 @@ class CollectionsEditor(QDialog):
     def __init__(self, parent=None):
         QDialog.__init__(self, parent)
 
+        win32_fix_title_bar_background(self)
+
         # Destroying the C++ object right after closing the dialog box,
         # otherwise it may be garbage-collected in another QThread
         # (e.g. the editor's analysis thread in Spyder), thus leading to
